/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.rest4j;

import com.rest4j.impl.PatchableType;
import com.rest4j.type.*;
import com.rest4j.json.JSONObject;

/**
 * @author Joseph Kapizza <joseph@rest4j.com>
 */
public interface Marshaller {
	Object marshal(ApiType apiType, Object content) throws ApiException;

	Object unmarshal(ApiType elementType, Object value) throws ApiException;

	Object unmarshalPatch(PatchableType type, Object original, JSONObject object) throws ApiException;

	ObjectApiType getObjectType(String model);

	ArrayApiType getArrayType(ApiType type);

	MapApiType getMapType(ApiType type);

	StringApiType getStringType(String[] values);

	NumberApiType getNumberType();

	BooleanApiType getBooleanType();

	DateApiType getDateType();
}
